@@ -21,9 +21,16 @@ from pathlib import Path
 
 import pandas as pd
 
+from mostlyai import engine
 from mostlyai.sdk._data.file.base import LocalFileContainer
 from mostlyai.sdk._data.file.utils import make_data_table_from_container
-from mostlyai.sdk._data.util.common import strip_column_prefix, TABLE_COLUMN_INFIX, TEMPORARY_PRIMARY_KEY
+from mostlyai.sdk._data.util.common import TABLE_COLUMN_INFIX, TEMPORARY_PRIMARY_KEY, strip_column_prefix
+from mostlyai.sdk._local.execution.plan import (
+    ExecutionPlan,
+    Task,
+    make_generator_execution_plan,
+    make_synthetic_dataset_execution_plan,
+)
 from mostlyai.sdk._local.execution.step_analyze_training_data import execute_step_analyze_training_data
 from mostlyai.sdk._local.execution.step_create_data_report import execute_step_create_data_report
 from mostlyai.sdk._local.execution.step_create_model_report import (
@@ -32,9 +39,9 @@ from mostlyai.sdk._local.execution.step_create_model_report import (
 from mostlyai.sdk._local.execution.step_deliver_data import execute_step_deliver_data
 from mostlyai.sdk._local.execution.step_encode_training_data import execute_step_encode_training_data
 from mostlyai.sdk._local.execution.step_finalize_generation import (
-    execute_step_finalize_generation,
     create_generation_schema,
-    update_total_rows_and_datapoints,
+    execute_step_finalize_generation,
+    update_total_rows,
 )
 from mostlyai.sdk._local.execution.step_generate_data import execute_step_generate_data
 from mostlyai.sdk._local.execution.step_generate_model_report_data import (
@@ -44,39 +51,39 @@ from mostlyai.sdk._local.execution.step_pull_training_data import (
     execute_step_pull_training_data,
 )
 from mostlyai.sdk._local.execution.step_train_model import execute_step_train_model
+from mostlyai.sdk._local.progress import LocalProgressCallback, get_current_utc_time
 from mostlyai.sdk._local.storage import (
-    read_generator_from_json,
-    write_generator_to_json,
+    get_model_label,
     read_connector_from_json,
+    read_generator_from_json,
     read_job_progress_from_json,
-    write_job_progress_to_json,
     read_synthetic_dataset_from_json,
+    write_generator_to_json,
+    write_job_progress_to_json,
     write_synthetic_dataset_to_json,
 )
 from mostlyai.sdk._local.synthetic_datasets import create_synthetic_dataset
 from mostlyai.sdk.domain import (
     ConnectorType,
     Generator,
-    StepCode,
     ModelType,
-    ProgressStatus,
-    SyntheticDataset,
-    TaskType,
     Probe,
+    ProgressStatus,
     SourceColumn,
+    StepCode,
+    SyntheticDataset,
     SyntheticProbeConfig,
+    TaskType,
 )
-from mostlyai.sdk._local.execution.plan import (
-    ExecutionPlan,
-    Task,
-    make_generator_execution_plan,
-    make_synthetic_dataset_execution_plan,
-)
-from mostlyai.sdk._local.progress import LocalProgressCallback, get_current_utc_time
 
 _LOG = logging.getLogger(__name__)
 
 
+def _set_random_state(random_state: int | None = None):
+    # rely on engine to set global random state
+    engine.set_random_state(random_state)
+
+
 def _move_training_artefacts(generator_dir: Path, job_workspace_dir: Path):
     for dir in ["Logs", "ModelStore", "ModelQAReports", "ModelQAStatistics"]:
         shutil.rmtree(generator_dir / dir, ignore_errors=True)
@@ -93,34 +100,47 @@ def _move_training_artefacts(generator_dir: Path, job_workspace_dir: Path):
 
 
 def _move_generation_artefacts(synthetic_dataset_dir: Path, job_workspace_dir: Path):
-    for dir in ["Logs", "RandomSamples", "ZIP", "DataQAReports"]:
-        (synthetic_dataset_dir / dir).mkdir(exist_ok=True)
     for path in job_workspace_dir.absolute().rglob("*"):
+        target = None
         if path.is_dir() and path.name == "RandomSamples":
-            path.rename(synthetic_dataset_dir / "RandomSamples")
-        if path.is_dir() and path.name == "ZIP":
-            path.rename(synthetic_dataset_dir / "ZIP")
-        if path.is_file() and path.parent.name == "DataQAReports":
-            path.rename(synthetic_dataset_dir / "DataQAReports" / path.name)
+            target = synthetic_dataset_dir / "RandomSamples"
+        elif path.is_dir() and path.name == "ZIP":
+            target = synthetic_dataset_dir / "ZIP"
+        elif path.is_file() and path.parent.name == "DataQAReports":
+            target = synthetic_dataset_dir / "DataQAReports" / path.name
+
+        if target:
+            if target.exists():
+                if target.is_dir():
+                    shutil.rmtree(target)
+                else:
+                    target.unlink()
+            target.parent.mkdir(parents=True, exist_ok=True)
+            shutil.move(str(path), str(target))
 
 
 def _mark_in_progress(resource: Generator | SyntheticDataset, resource_dir: Path):
     if isinstance(resource, Generator):
         resource.training_status = ProgressStatus.in_progress
+        write_resource_to_json = write_generator_to_json
     else:
         resource.generation_status = ProgressStatus.in_progress
+        write_resource_to_json = write_synthetic_dataset_to_json
     job_progress = read_job_progress_from_json(resource_dir)
     job_progress.status = ProgressStatus.in_progress
     for step in job_progress.steps:
         step.status = ProgressStatus.in_progress
+    write_resource_to_json(resource_dir, resource)
     write_job_progress_to_json(resource_dir, job_progress)
 
 
 def _mark_done(resource: Generator | SyntheticDataset, resource_dir: Path):
     if isinstance(resource, Generator):
         resource.training_status = ProgressStatus.done
+        write_resource_to_json = write_generator_to_json
     else:
         resource.generation_status = ProgressStatus.done
+        write_resource_to_json = write_synthetic_dataset_to_json
     now = get_current_utc_time()
     job_progress = read_job_progress_from_json(resource_dir)
     job_progress.status = ProgressStatus.done
@@ -132,30 +152,40 @@ def _mark_done(resource: Generator | SyntheticDataset, resource_dir: Path):
         step.start_date = step.start_date or now
         step.end_date = step.end_date or now
         step.progress.value = step.progress.max
+    write_resource_to_json(resource_dir, resource)
     write_job_progress_to_json(resource_dir, job_progress)
 
 
 def _mark_failed(resource: Generator | SyntheticDataset, resource_dir: Path):
     if isinstance(resource, Generator):
         resource.training_status = ProgressStatus.failed
+        write_resource_to_json = write_generator_to_json
     else:
         resource.generation_status = ProgressStatus.failed
+        write_resource_to_json = write_synthetic_dataset_to_json
     job_progress = read_job_progress_from_json(resource_dir)
     job_progress.status = ProgressStatus.failed
     for step in job_progress.steps:
         if step.status != ProgressStatus.done:
             step.status = ProgressStatus.failed
+    write_resource_to_json(resource_dir, resource)
     write_job_progress_to_json(resource_dir, job_progress)
 
 
-def _copy_model(generator_dir: Path, model_label: str, workspace_dir: Path):
+def _copy_model(generator_dir: Path, model_label: str, workspace_dir: Path) -> bool:
     model_path = generator_dir / "ModelStore" / model_label
-    shutil.copytree(model_path, workspace_dir / "ModelStore")
+    if model_path.exists():
+        shutil.copytree(model_path, workspace_dir / "ModelStore")
+        return True
+    return False
 
 
-def _copy_statistics(generator_dir: Path, model_label: str, workspace_dir: Path):
+def _copy_statistics(generator_dir: Path, model_label: str, workspace_dir: Path) -> bool:
     statistics_path = generator_dir / "ModelQAStatistics" / model_label
-    shutil.copytree(statistics_path, workspace_dir / "ModelQAStatistics")
+    if statistics_path.exists():
+        shutil.copytree(statistics_path, workspace_dir / "ModelQAStatistics")
+        return True
+    return False
 
 
 def _set_overall_accuracy(generator: Generator) -> None:
@@ -278,17 +308,18 @@ class Execution:
                 if t.configuration.sample_seed_connector_id is not None
             ]
         connectors_dir = self._home_dir / "connectors"
-        for connector_path in connectors_dir.iterdir():
-            if not (connector_path.name in connector_ids and connector_path.is_dir()):
-                continue
+        if connectors_dir.exists():
+            for connector_path in connectors_dir.iterdir():
+                if not (connector_path.name in connector_ids and connector_path.is_dir()):
+                    continue
 
-            try:
-                connector = read_connector_from_json(connector_path)
-                if connector.type == ConnectorType.file_upload:
-                    shutil.rmtree(connector_path)
-            except Exception as e:
-                _LOG.info(f"Failed to clear connector {connector_path}: {e}")
-                pass
+                try:
+                    connector = read_connector_from_json(connector_path)
+                    if connector.type == ConnectorType.file_upload:
+                        shutil.rmtree(connector_path)
+                except Exception as e:
+                    _LOG.info(f"Failed to clear connector {connector_path}: {e}")
+                    pass
 
     def run(self):
         # execute job
@@ -297,12 +328,8 @@ class Execution:
             TaskType.train_tabular: self.execute_task_train,
             TaskType.train_language: self.execute_task_train,
             TaskType.finalize_training: self.execute_task_finalize_training,
-            TaskType.generate_tabular: self.execute_task_generate,
-            TaskType.generate_language: self.execute_task_generate,
-            TaskType.finalize_generation: self.execute_task_finalize_generation,
-            TaskType.probe_tabular: self.execute_task_generate,
-            TaskType.probe_language: self.execute_task_generate,
-            TaskType.finalize_probing: self.execute_task_finalize_probing,
+            TaskType.generate: self.execute_task_generate,
+            TaskType.probe: self.execute_task_generate,
         }
         for task in self._execution_plan.tasks:
             handler = task_handlers.get(task.type)
@@ -316,17 +343,17 @@ class Execution:
     def execute_task_train(self, task: Task):
         # gather arguments that are common across steps
         generator = self._generator
-        model_type = ModelType.tabular if task.type == TaskType.train_tabular else ModelType.language
-        model_label = f"{task.target_table_name}:{model_type.value.lower()}"
         tgt_table = next(t for t in generator.tables if t.name == task.target_table_name)
+        model_type = ModelType.tabular if task.type == TaskType.train_tabular else ModelType.language
+        model_label = get_model_label(tgt_table, model_type, path_safe=False)
+        model_label_path = get_model_label(tgt_table, model_type, path_safe=True)
         generator_dir = self._home_dir / "generators" / generator.id
-        workspace_dir = self._job_workspace_dir / model_label
+        workspace_dir = self._job_workspace_dir / model_label_path
         workspace_dir.mkdir(parents=True, exist_ok=True)
         update_progress_fn = partial(LocalProgressCallback, resource_path=generator_dir, model_label=model_label)
 
-        # if training shall be continued, then let's first copy the ModelStore
-        if generator.training_status == ProgressStatus.continue_:
-            _copy_model(generator_dir=generator_dir, model_label=model_label, workspace_dir=workspace_dir)
+        # let's first copy the ModelStore in case that it already exists (training continuation)
+        _copy_model(generator_dir=generator_dir, model_label=model_label_path, workspace_dir=workspace_dir)
 
         # step: PULL_TRAINING_DATA
         connectors = [
@@ -358,7 +385,7 @@ class Execution:
         for column in tgt_table.columns:
             if column.name in encoding_types:
                 column.model_encoding_type = encoding_types[column.name]
-        # update generator with column-level value ranges, so that these are known for conditional generation
+        # update generator with column-level value ranges, so that these are known for conditional simulations
         for column in tgt_table.columns:
             if column.name in value_ranges:
                 column.value_range = value_ranges[column.name]
@@ -380,106 +407,137 @@ class Execution:
             upload_model_data_callback=None,  # not applicable in local mode
         )
 
-        # step: GENERATE_MODEL_REPORT_DATA
-        execute_step_generate_model_report_data(
-            workspace_dir=workspace_dir,
-            model_type=model_type,
-            update_progress=update_progress_fn(step_code=StepCode.generate_model_report_data),
-        )
+        step_codes = [s.step_code for s in task.steps]
+        if StepCode.generate_model_report_data in step_codes:
+            # step: GENERATE_MODEL_REPORT_DATA
+            execute_step_generate_model_report_data(
+                workspace_dir=workspace_dir,
+                model_type=model_type,
+                update_progress=update_progress_fn(step_code=StepCode.generate_model_report_data),
+            )
 
-        # step: CREATE_MODEL_REPORT
-        model_metrics = execute_step_create_model_report(
-            generator=generator,
-            model_type=model_type,
-            target_table_name=task.target_table_name,
-            workspace_dir=workspace_dir,
-            update_progress=update_progress_fn(step_code=StepCode.create_model_report),
-        )
-        # update generator with model metrics
-        if model_type == ModelType.tabular:
-            tgt_table.tabular_model_metrics = model_metrics
-        else:
-            tgt_table.language_model_metrics = model_metrics
+            # step: CREATE_MODEL_REPORT
+            model_metrics = execute_step_create_model_report(
+                generator=generator,
+                model_type=model_type,
+                target_table_name=task.target_table_name,
+                workspace_dir=workspace_dir,
+                update_progress=update_progress_fn(step_code=StepCode.create_model_report),
+            )
+            # update generator with model metrics
+            if model_type == ModelType.tabular:
+                tgt_table.tabular_model_metrics = model_metrics
+            else:
+                tgt_table.language_model_metrics = model_metrics
 
     def execute_task_finalize_training(self, task: Task):
         pass
 
     def execute_task_generate(self, task: Task):
-        # gather common step arguments
         generator = self._generator
         synthetic_dataset = self._synthetic_dataset
-        is_probe = task.type in (TaskType.probe_tabular, TaskType.probe_language)
-        model_type = (
-            ModelType.tabular
-            if task.type in (TaskType.generate_tabular, TaskType.probe_tabular)
-            else ModelType.language
-        )
-        model_label = f"{task.target_table_name}:{model_type.value.lower()}"
-        sd_table = next(t for t in synthetic_dataset.tables if t.name == task.target_table_name)
         synthetic_dataset_dir = self._home_dir / "synthetic-datasets" / synthetic_dataset.id
         generator_dir = self._home_dir / "generators" / generator.id
-        workspace_dir = self._job_workspace_dir / model_label
-        workspace_dir.mkdir()
-        update_progress_fn = partial(
-            LocalProgressCallback, resource_path=synthetic_dataset_dir, model_label=model_label
-        )
 
-        # copy AI model to workspace
-        _copy_model(generator_dir=generator_dir, model_label=model_label, workspace_dir=workspace_dir)
+        visited_tables = set()
+        table_lookup = {table.name: table for table in synthetic_dataset.tables}
 
-        # step: GENERATE_DATA
-        if sd_table.configuration.sample_seed_connector_id is not None:
-            sample_seed = _fetch_sample_seed(
-                home_dir=self._home_dir,
-                connector_id=sd_table.configuration.sample_seed_connector_id,
+        for step in task.steps:
+            model_type = (
+                ModelType.tabular
+                if step.step_code in {StepCode.generate_data_tabular, StepCode.create_data_report_tabular}
+                else ModelType.language
             )
-        else:
-            sample_seed = None
-        schema = create_generation_schema(
-            generator=generator,
-            job_workspace_dir=self._job_workspace_dir,
-            step="pull_context_data",
-        )
-        execute_step_generate_data(
-            generator=generator,
-            synthetic_dataset=synthetic_dataset,
-            target_table_name=task.target_table_name,
-            model_type=model_type,
-            sample_seed=sample_seed,
-            schema=schema,
-            workspace_dir=workspace_dir,
-            update_progress=update_progress_fn(step_code=StepCode.generate_data),
-        )
+            model_label = get_model_label(step.target_table_name, model_type, path_safe=False)
+            model_label_path = get_model_label(step.target_table_name, model_type, path_safe=True)
+            workspace_dir = self._job_workspace_dir / model_label_path
+            workspace_dir.mkdir(exist_ok=True)
 
-        if not is_probe:
-            # copy QA statistics to workspace
-            _copy_statistics(generator_dir=generator_dir, model_label=model_label, workspace_dir=workspace_dir)
-            # step: GENERATE_DATA_REPORT
-            execute_step_create_data_report(
-                generator=generator,
-                target_table_name=task.target_table_name,
-                model_type=model_type,
-                workspace_dir=workspace_dir,
-                update_progress=update_progress_fn(step_code=StepCode.create_data_report),
+            update_progress = partial(
+                LocalProgressCallback, resource_path=synthetic_dataset_dir, model_label=model_label
             )
 
-        # as a last step of LANGUAGE model generation, merge context and generated data
-        if model_type == ModelType.language:
-            _merge_tabular_language_data(workspace_dir=workspace_dir)
-            # overwrite tabular SyntheticData with tabular+language SyntheticData
-            tabular_workspace_dir = self._job_workspace_dir / f"{task.target_table_name}:tabular"
-            tabular_workspace_dir.mkdir(parents=True, exist_ok=True)
-            shutil.rmtree(tabular_workspace_dir / "SyntheticData", ignore_errors=True)
-            shutil.move(workspace_dir / "SyntheticData", tabular_workspace_dir / "SyntheticData")
+            if step.step_code in {StepCode.generate_data_tabular, StepCode.generate_data_language}:
+                # step: GENERATE_DATA
+                _copy_model(generator_dir=generator_dir, model_label=model_label_path, workspace_dir=workspace_dir)
 
-    def execute_task_finalize_generation(self, task: Task):
+                visited_tables.add(step.target_table_name)
+
+                table = table_lookup[step.target_table_name]
+                sample_seed = (
+                    _fetch_sample_seed(
+                        home_dir=self._home_dir, connector_id=table.configuration.sample_seed_connector_id
+                    )
+                    if table.configuration.sample_seed_connector_id
+                    else None
+                )
+
+                schema = create_generation_schema(
+                    generator=generator,
+                    job_workspace_dir=self._job_workspace_dir,
+                    step="pull_context_data",
+                )
+
+                execute_step_generate_data(
+                    generator=generator,
+                    synthetic_dataset=synthetic_dataset,
+                    target_table_name=step.target_table_name,
+                    model_type=model_type,
+                    sample_seed=sample_seed,
+                    schema=schema,
+                    workspace_dir=workspace_dir,
+                    update_progress=update_progress(step_code=step.step_code),
+                )
+
+            elif step.step_code in {StepCode.create_data_report_tabular, StepCode.create_data_report_language}:
+                model_report_available = _copy_statistics(
+                    generator_dir=generator_dir, model_label=model_label_path, workspace_dir=workspace_dir
+                )
+                if not model_report_available:
+                    continue
+                # step: GENERATE_DATA_REPORT
+                execute_step_create_data_report(
+                    generator=generator,
+                    target_table_name=step.target_table_name,
+                    model_type=model_type,
+                    workspace_dir=workspace_dir,
+                    update_progress=update_progress(step_code=step.step_code),
+                )
+
+            elif step.step_code in {StepCode.finalize_generation, StepCode.finalize_probing}:
+                # for every LANGUAGE model generation, merge context and generated data
+                for table_name in visited_tables:
+                    language_path = self._job_workspace_dir / get_model_label(
+                        table_lookup[table_name], ModelType.language, path_safe=True
+                    )
+                    if language_path.exists():
+                        _merge_tabular_language_data(workspace_dir=language_path)
+
+                        tabular_workspace = self._job_workspace_dir / get_model_label(
+                            table_lookup[table_name], ModelType.tabular, path_safe=True
+                        )
+                        tabular_workspace.mkdir(parents=True, exist_ok=True)
+                        shutil.rmtree(tabular_workspace / "SyntheticData", ignore_errors=True)
+                        shutil.move(language_path / "SyntheticData", tabular_workspace / "SyntheticData")
+
+                finalize_method = (
+                    self.execute_finalize_generation
+                    if step.step_code == StepCode.finalize_generation
+                    else self.execute_finalize_probing
+                )
+                finalize_method()
+
+            elif step.step_code == StepCode.deliver_data:
+                self.execute_deliver_data()
+
+    def execute_finalize_generation(self):
         schema = create_generation_schema(
             generator=self._generator,
             job_workspace_dir=self._job_workspace_dir,
             step="finalize_generation",
         )
-        # step: FINALIZE_GENERATION
-        usage = execute_step_finalize_generation(
+
+        usages = execute_step_finalize_generation(
             schema=schema,
             is_probe=False,
             job_workspace_dir=self._job_workspace_dir,
@@ -489,20 +547,22 @@ class Execution:
                 step_code=StepCode.finalize_generation,
             ),
         )
-        # update total rows and datapoints
-        update_total_rows_and_datapoints(self._synthetic_dataset, usage)
+        update_total_rows(self._synthetic_dataset, usages)
 
-        # step: DELIVER_DATA
+    def execute_deliver_data(self):
         schema = create_generation_schema(
             generator=self._generator,
             job_workspace_dir=self._job_workspace_dir,
             step="deliver_data",
         )
+
         delivery = self._synthetic_dataset.delivery
-        if delivery is not None and delivery.destination_connector_id is not None:
-            connector = read_connector_from_json(self._home_dir / "connectors" / delivery.destination_connector_id)
-        else:
-            connector = None
+        connector = (
+            read_connector_from_json(self._home_dir / "connectors" / delivery.destination_connector_id)
+            if delivery and delivery.destination_connector_id
+            else None
+        )
+
         execute_step_deliver_data(
             generator=self._generator,
             delivery=delivery,
@@ -511,14 +571,14 @@ class Execution:
             job_workspace_dir=self._job_workspace_dir,
         )
 
-    def execute_task_finalize_probing(self, task: Task):
+    def execute_finalize_probing(self):
         schema = create_generation_schema(
             generator=self._generator,
             job_workspace_dir=self._job_workspace_dir,
             step="finalize_generation",
         )
         # step: FINALIZE_GENERATION
-        execute_step_finalize_generation(
+        _ = execute_step_finalize_generation(
             schema=schema,
             is_probe=True,
             job_workspace_dir=self._job_workspace_dir,
@@ -531,9 +591,11 @@ class Execution:
 def execute_training_job(generator_id: str, home_dir: Path):
     generator_dir = home_dir / "generators" / generator_id
     generator = read_generator_from_json(generator_dir)
+    _set_random_state(generator.random_state)
     if generator.training_status not in [ProgressStatus.new, ProgressStatus.continue_]:
         raise ValueError("Generator has already been trained")
     _mark_in_progress(resource=generator, resource_dir=generator_dir)
+
     # PLAN
     plan = make_generator_execution_plan(generator)
     # EXECUTE
@@ -545,14 +607,15 @@ def execute_training_job(generator_id: str, home_dir: Path):
         _move_training_artefacts(generator_dir=generator_dir, job_workspace_dir=execution._job_workspace_dir)
         # flag as DONE
         _mark_done(resource=generator, resource_dir=generator_dir)
-    except Exception:
+    except Exception as e:
         _mark_failed(resource=generator, resource_dir=generator_dir)
         _LOG.error(traceback.format_exc())
+        _LOG.error(f"Exception type: {type(e)}")
+        _LOG.error(f"Exception message: {e}")
         raise
     finally:
         execution.clear_job_workspace()
         execution.clear_file_upload_connectors()
-        write_generator_to_json(generator_dir, generator)
 
     try:
         _probe_random_samples(home_dir=home_dir, generator=generator)
@@ -564,16 +627,17 @@ def execute_training_job(generator_id: str, home_dir: Path):
 def execute_generation_job(synthetic_dataset_id: str, home_dir: Path):
     synthetic_dataset_dir = home_dir / "synthetic-datasets" / synthetic_dataset_id
     synthetic_dataset = read_synthetic_dataset_from_json(synthetic_dataset_dir)
+    _set_random_state(synthetic_dataset.random_state)
     generator_dir = home_dir / "generators" / synthetic_dataset.generator_id
     generator = read_generator_from_json(generator_dir)
     if generator.training_status != ProgressStatus.done:
         raise ValueError("Generator has not been trained yet")
     if synthetic_dataset.generation_status != ProgressStatus.new:
         raise ValueError("Synthetic Dataset has already been generated")
-
     _mark_in_progress(resource=synthetic_dataset, resource_dir=synthetic_dataset_dir)
+
     # PLAN
-    plan = make_synthetic_dataset_execution_plan(generator)
+    plan = make_synthetic_dataset_execution_plan(generator, synthetic_dataset)
     # EXECUTE
     execution = Execution(
         execution_plan=plan,
@@ -596,18 +660,18 @@ def execute_generation_job(synthetic_dataset_id: str, home_dir: Path):
     finally:
         execution.clear_job_workspace()
         execution.clear_file_upload_connectors()
-        write_synthetic_dataset_to_json(synthetic_dataset_dir, synthetic_dataset)
 
 
 def execute_probing_job(synthetic_dataset_id: str, home_dir: Path) -> list[Probe]:
     synthetic_dataset_dir = home_dir / "synthetic-datasets" / synthetic_dataset_id
     synthetic_dataset = read_synthetic_dataset_from_json(synthetic_dataset_dir)
+    _set_random_state(synthetic_dataset.random_state)
     generator_id = synthetic_dataset.generator_id
     generator_dir = home_dir / "generators" / generator_id
     generator = read_generator_from_json(generator_dir)
 
     # PLAN
-    plan = make_synthetic_dataset_execution_plan(generator, is_probe=True)
+    plan = make_synthetic_dataset_execution_plan(generator, synthetic_dataset, is_probe=True)
     # EXECUTE
     execution = Execution(
         execution_plan=plan, generator=generator, synthetic_dataset=synthetic_dataset, home_dir=home_dir
