@@ -13,8 +13,10 @@
 # limitations under the License.
 
 
-from pathlib import Path
 from collections.abc import Callable
+from pathlib import Path
+
+from mostlyai.sdk._local.execution.migration import migrate_workspace
 
 
 def execute_step_encode_training_data(
@@ -25,6 +27,9 @@ def execute_step_encode_training_data(
     # import ENGINE here to avoid pre-mature loading of large ENGINE dependencies
     from mostlyai import engine
 
+    # ensure backward compatibility
+    migrate_workspace(workspace_dir)
+
     # call ENCODE
     engine.encode(
         workspace_dir=workspace_dir,
