@@ -15,59 +15,72 @@ import os
 import shutil
 import subprocess
 import sys
+import tempfile
 import uuid
 import zipfile
 from io import BytesIO
 from pathlib import Path
 
-from fastapi import APIRouter, Body, HTTPException, UploadFile, File
+from fastapi import APIRouter, Body, File, Form, HTTPException, UploadFile
 from fastapi.encoders import jsonable_encoder
-from fastapi.responses import FileResponse, JSONResponse, StreamingResponse, HTMLResponse
+from fastapi.responses import FileResponse, HTMLResponse, JSONResponse, RedirectResponse, StreamingResponse
+from starlette.background import BackgroundTask
 
 from mostlyai import sdk
 from mostlyai.sdk._data.conversions import create_container_from_connector
-from mostlyai.sdk._local import generators, synthetic_datasets
+from mostlyai.sdk._data.file.utils import read_data_table_from_path
+from mostlyai.sdk._local import connectors, generators, synthetic_datasets
 from mostlyai.sdk._local.execution.jobs import execute_probing_job
 from mostlyai.sdk._local.generators import create_generator as create_generator_model
-from mostlyai.sdk._local import connectors
+from mostlyai.sdk._local.storage import (
+    convert_model_label_path,
+    create_zip_in_memory,
+    get_model_label,
+    read_connector_from_json,
+    read_dataset_from_json,
+    read_generator_from_json,
+    read_job_progress_from_json,
+    read_synthetic_dataset_from_json,
+    write_connector_to_json,
+    write_dataset_to_json,
+    write_generator_to_json,
+    write_synthetic_dataset_to_json,
+)
 from mostlyai.sdk.domain import (
     AboutService,
-    ConnectorType,
-    CurrentUser,
-    Generator,
-    GeneratorPatchConfig,
-    GeneratorListItem,
-    GeneratorConfig,
-    ProgressStatus,
-    JobProgress,
-    ConnectorConfig,
-    Connector,
-    ConnectorListItem,
-    SyntheticDataset,
-    SyntheticDatasetConfig,
-    SyntheticDatasetPatchConfig,
-    SyntheticDatasetFormat,
-    ConnectorPatchConfig,
     ComputeListItem,
-    ModelType,
-    SyntheticProbeConfig,
-    Probe,
+    Connector,
+    ConnectorConfig,
+    ConnectorDeleteDataConfig,
+    ConnectorListItem,
+    ConnectorPatchConfig,
+    ConnectorReadDataConfig,
+    ConnectorType,
+    ConnectorWriteDataConfig,
+    CurrentUser,
+    Dataset,
+    DatasetConfig,
+    DatasetListItem,
+    DatasetPatchConfig,
+    Generator,
     GeneratorCloneConfig,
     GeneratorCloneTrainingStatus,
-    SyntheticDatasetReportType,
+    GeneratorConfig,
+    GeneratorListItem,
+    GeneratorPatchConfig,
+    IfExists,
+    JobProgress,
+    ModelType,
+    Probe,
+    ProgressStatus,
+    SyntheticDataset,
+    SyntheticDatasetConfig,
+    SyntheticDatasetFormat,
     SyntheticDatasetListItem,
+    SyntheticDatasetPatchConfig,
+    SyntheticDatasetReportType,
+    SyntheticProbeConfig,
 )
-from mostlyai.sdk._local.storage import (
-    read_generator_from_json,
-    write_generator_to_json,
-    read_job_progress_from_json,
-    read_connector_from_json,
-    read_synthetic_dataset_from_json,
-    write_synthetic_dataset_to_json,
-    create_zip_in_memory,
-    write_connector_to_json,
-)
-from mostlyai.sdk._data.file.utils import read_data_table_from_path
 
 
 class Routes:
@@ -104,6 +117,10 @@ class Routes:
                 """
 
     def _initialize_routes(self):
+        @self.router.get("/", include_in_schema=False)
+        async def root() -> RedirectResponse:
+            return RedirectResponse(url="/docs")
+
         ## GENERAL
 
         @self.router.get("/about", response_model=AboutService)
@@ -121,27 +138,32 @@ class Routes:
         @self.router.get("/models/{model_type}")
         async def list_models(model_type: str) -> JSONResponse:
             if model_type == ModelType.tabular:
-                return JSONResponse(content=["MOSTLY_AI/Small", "MOSTLY_AI/Medium", "MOSTLY_AI/Large"])
+                models = ["MOSTLY_AI/Small", "MOSTLY_AI/Medium", "MOSTLY_AI/Large"]
             elif model_type == ModelType.language:
-                return JSONResponse(
-                    content=["MOSTLY_AI/LSTMFromScratch-3m", "microsoft/phi-1_5", "(HuggingFace-hosted models)"]
-                )
-            return JSONResponse(content=[])
+                models = ["MOSTLY_AI/LSTMFromScratch-3m", "microsoft/phi-1_5", "(HuggingFace-hosted models)"]
+            else:
+                models = []
+            return JSONResponse(content=models)
 
         ## CONNECTORS
 
         @self.router.get("/connectors")
         async def list_connectors(
-            offset=0, limit=50, searchTerm: str | None = None, access_type: str | None = None
+            offset=0,
+            limit=50,
+            searchTerm: str | None = None,
+            access_type: str | None = None,
         ) -> JSONResponse:
             connector_dirs = [p for p in (self.home_dir / "connectors").glob("*") if p.is_dir()]
             connector_list_items = []
             for connector_dir in connector_dirs:
                 connector = read_connector_from_json(connector_dir)
-                if not searchTerm or searchTerm.lower() in (connector.name or "").lower():
-                    if not access_type or access_type == connector.access_type:
-                        # use model_construct to skip validation and warnings of extra fields
-                        connector_list_items.append(ConnectorListItem.model_construct(**connector.model_dump()))
+                connector_string = " ".join([connector.name or "", connector.description or ""]).lower()
+                if searchTerm and searchTerm.lower() not in connector_string:
+                    continue
+                if access_type and access_type != connector.access_type:
+                    continue
+                connector_list_items.append(ConnectorListItem.model_construct(**connector.model_dump()))
 
             # use jsonable_encoder to handle non-serializable objects like datetime
             return JSONResponse(
@@ -162,8 +184,6 @@ class Routes:
         @self.router.get("/connectors/{id}", response_model=Connector)
         async def get_connector(id: str) -> Connector:
             connector_dir = self.home_dir / "connectors" / id
-            if not connector_dir.exists():
-                raise HTTPException(status_code=404, detail=f"Connector `{id}` not found")
             connector = read_connector_from_json(connector_dir)
             return connector
 
@@ -220,28 +240,175 @@ class Routes:
             ]
             return JSONResponse(status_code=200, content=columns)
 
+        @self.router.post("/connectors/{id}/read-data")
+        async def read_data(id: str, config: ConnectorReadDataConfig = Body(...)) -> StreamingResponse:
+            connector_dir = self.home_dir / "connectors" / id
+            connector = read_connector_from_json(connector_dir)
+            df = connectors.read_data_from_connector(connector, config)
+
+            # this file can only be safely removed once the streaming response is complete
+            with tempfile.NamedTemporaryFile(suffix=".parquet", delete=False) as tmp_file:
+                tmp_path = tmp_file.name
+                df.to_parquet(tmp_path)
+
+            return StreamingResponse(
+                open(tmp_path, mode="rb"),
+                media_type="application/octet-stream",
+                headers={"Content-Disposition": "attachment; filename=data.parquet"},
+                background=BackgroundTask(Path(tmp_path).unlink, missing_ok=True),
+            )
+
+        @self.router.post("/connectors/{id}/write-data")
+        async def write_data(
+            id: str,
+            file: UploadFile | None = File(None),
+            location: str = Form(...),
+            if_exists: IfExists = Form("FAIL", alias="ifExists"),
+        ) -> None:
+            connector_dir = self.home_dir / "connectors" / id
+            connector = read_connector_from_json(connector_dir)
+            file_content = await file.read() if file else None
+            config = ConnectorWriteDataConfig(location=location, file=file_content, if_exists=if_exists)
+            connectors.write_data_to_connector(connector, config)
+
+        @self.router.post("/connectors/{id}/delete-data")
+        async def delete_data(id: str, config: ConnectorDeleteDataConfig = Body(...)) -> None:
+            connector_dir = self.home_dir / "connectors" / id
+            connector = read_connector_from_json(connector_dir)
+            connectors.delete_data_from_connector(connector, config)
+
+        @self.router.post("/connectors/{id}/query")
+        async def query(id: str, sql: str = Body(..., embed=True)) -> StreamingResponse:
+            connector_dir = self.home_dir / "connectors" / id
+            connector = read_connector_from_json(connector_dir)
+            df = connectors.query_data_from_connector(connector, sql)
+
+            # this file can only be safely removed once the streaming response is complete
+            with tempfile.NamedTemporaryFile(suffix=".parquet", delete=False) as tmp_file:
+                tmp_path = tmp_file.name
+                df.to_parquet(tmp_path)
+
+            return StreamingResponse(
+                open(tmp_path, mode="rb"),
+                media_type="application/octet-stream",
+                headers={"Content-Disposition": "attachment; filename=query_result.parquet"},
+                background=BackgroundTask(Path(tmp_path).unlink, missing_ok=True),
+            )
+
+        ## DATASETS
+
+        @self.router.get("/datasets")
+        async def list_datasets(
+            offset: int = 0,
+            limit: int = 50,
+            searchTerm: str | None = None,
+        ) -> JSONResponse:
+            dataset_dirs = [p for p in (self.home_dir / "datasets").glob("*") if p.is_dir()]
+            dataset_list_items = []
+            for dataset_dir in dataset_dirs:
+                dataset = read_dataset_from_json(dataset_dir)
+                dataset_string = " ".join([dataset.name or "", dataset.description or ""]).lower()
+                if searchTerm and searchTerm.lower() not in dataset_string:
+                    continue
+                # use model_construct to skip validation and warnings of extra fields
+                dataset_list_items.append(DatasetListItem.model_construct(**dataset.model_dump()))
+
+            return JSONResponse(
+                status_code=200,
+                content=jsonable_encoder(
+                    {
+                        "totalCount": len(dataset_list_items),
+                        "results": dataset_list_items[int(offset) : int(offset) + int(limit)],
+                    }
+                ),
+            )
+
+        @self.router.post("/datasets", response_model=Dataset)
+        async def create_dataset(config: DatasetConfig = Body(...)) -> Dataset:
+            dataset = Dataset(**config.model_dump())
+            dataset_dir = self.home_dir / "datasets" / dataset.id
+            write_dataset_to_json(dataset_dir, dataset)
+            return dataset
+
+        @self.router.get("/datasets/{id}", response_model=Dataset)
+        async def get_dataset(id: str) -> Dataset:
+            dataset_dir = self.home_dir / "datasets" / id
+            if not dataset_dir.exists():
+                raise HTTPException(status_code=404, detail=f"Dataset `{id}` not found")
+            dataset = read_dataset_from_json(dataset_dir)
+            return dataset
+
+        @self.router.patch("/datasets/{id}", response_model=Dataset)
+        async def patch_dataset(id: str, config: DatasetPatchConfig = Body(...)) -> Dataset:
+            dataset_dir = self.home_dir / "datasets" / id
+            dataset = read_dataset_from_json(dataset_dir)
+            for key, value in config.model_dump().items():
+                if value is not None:
+                    setattr(dataset, key, value)
+            write_dataset_to_json(dataset_dir, dataset)
+            return dataset
+
+        @self.router.delete("/datasets/{id}")
+        async def delete_dataset(id: str):
+            dataset_dir = self.home_dir / "datasets" / id
+            shutil.rmtree(dataset_dir, ignore_errors=True)
+
+        @self.router.get("/datasets/{id}/file")
+        async def download_dataset_file(id: str, filepath: str) -> FileResponse:
+            dataset_dir = self.home_dir / "datasets" / id
+            filename = Path(filepath).name
+            return StreamingResponse(
+                open(dataset_dir / filepath, "rb"),
+                media_type="application/octet-stream",
+                headers={"Content-Disposition": f"attachment; filename={filename}"},
+            )
+
+        @self.router.post("/datasets/{id}/file")
+        async def upload_dataset_file(id: str, file: UploadFile = File(...)) -> None:
+            dataset_dir = self.home_dir / "datasets" / id
+            dataset = read_dataset_from_json(dataset_dir)
+            try:
+                file_content = await file.read()
+                with open(dataset_dir / file.filename, "wb") as f:
+                    f.write(file_content)
+                dataset.files.append(file.filename)
+                write_dataset_to_json(dataset_dir, dataset)
+            except Exception as e:
+                raise HTTPException(status_code=500, detail=f"Error uploading file `{file.filename}`: {e}")
+
+        @self.router.delete("/datasets/{id}/file")
+        async def delete_dataset_file(id: str, filepath: str) -> None:
+            dataset_dir = self.home_dir / "datasets" / id
+            dataset = read_dataset_from_json(dataset_dir)
+            if os.path.exists(dataset_dir / filepath):
+                os.remove(dataset_dir / filepath)
+            dataset.files = [f for f in dataset.files if f != filepath]
+            write_dataset_to_json(dataset_dir, dataset)
+
         ## GENERATORS
 
         @self.router.get("/generators")
         async def list_generators(
-            offset: int = 0, limit: int = 50, searchTerm: str | None = None, status: str | None = None
+            offset: int = 0,
+            limit: int = 50,
+            searchTerm: str | None = None,
+            status: str | None = None,
         ) -> JSONResponse:
             generator_dirs = [p for p in (self.home_dir / "generators").glob("*") if p.is_dir()]
             generator_list_items = []
             for generator_dir in generator_dirs:
                 generator = read_generator_from_json(generator_dir)
-                if (
-                    not searchTerm
-                    or searchTerm.lower() in (generator.name or "").lower()
-                    or searchTerm.lower() in (generator.description or "").lower()
-                ):
-                    if not status or status == generator.training_status:
-                        # use model_construct to skip validation and warnings of extra fields
-                        generator_list_items.append(GeneratorListItem.model_construct(**generator.model_dump()))
-            # use jsonable_encoder to handle non-serializable objects like datetime
+                generator_string = " ".join([generator.name or "", generator.description or ""]).lower()
+                if searchTerm and searchTerm.lower() not in generator_string:
+                    continue
+                if status and status != generator.training_status:
+                    continue
+                # use model_construct to skip validation and warnings of extra fields
+                generator_list_items.append(GeneratorListItem.model_construct(**generator.model_dump()))
+
             return JSONResponse(
                 status_code=200,
-                content=jsonable_encoder(
+                content=jsonable_encoder(  # use jsonable_encoder to handle non-serializable objects like datetime
                     {
                         "totalCount": len(generator_list_items),
                         "results": generator_list_items[int(offset) : int(offset) + int(limit)],
@@ -298,6 +465,7 @@ class Routes:
             if any(read_connector_from_json(c).type == ConnectorType.file_upload for c in connector_dirs):
                 raise HTTPException(status_code=400, detail="Cannot clone a generator with uploaded files.")
             generator_config = generators.get_generator_config(self.home_dir, id)
+            generator_config.name = f"Clone - {generator_config.name}"
             new_generator = create_generator_model(home_dir=self.home_dir, config=generator_config)
             new_generator_dir = self.home_dir / "generators" / new_generator.id
             if config.training_status == GeneratorCloneTrainingStatus.continue_:
@@ -311,8 +479,8 @@ class Routes:
             generator = read_generator_from_json(generator_dir)
             table = next((t for t in generator.tables if t.id == table_id), None)
             reports_dir = self.home_dir / "generators" / id / "ModelQAReports"
-            fn = reports_dir / f"{table.name}:{modelType.lower()}.html"
-            return HTMLResponse(content=fn.read_text())
+            fn = reports_dir / f"{get_model_label(table, modelType, path_safe=True)}.html"
+            return HTMLResponse(content=fn.read_text(encoding="utf-8"))
 
         @self.router.get("/generators/{id}/training", response_model=JobProgress)
         async def get_training_progress(id: str) -> JobProgress:
@@ -367,7 +535,10 @@ class Routes:
             file_content = await file.read()
             try:
                 with zipfile.ZipFile(BytesIO(file_content)) as zip_ref:
-                    zip_ref.extractall(generator_dir)
+                    for zip_info in zip_ref.filelist:
+                        zip_info.filename = convert_model_label_path(zip_info.filename)
+                        zip_ref.extract(zip_info, generator_dir)
+
             except zipfile.BadZipFile:
                 raise HTTPException(status_code=400, detail="Invalid ZIP file")
 
@@ -387,26 +558,30 @@ class Routes:
 
         @self.router.get("/synthetic-datasets")
         async def list_synthetic_datasets(
-            offset: int = 0, limit: int = 50, searchTerm: str | None = None, status: str | None = None
+            offset: int = 0,
+            limit: int = 50,
+            searchTerm: str | None = None,
+            status: str | None = None,
         ) -> JSONResponse:
             synthetic_dataset_dirs = [p for p in (self.home_dir / "synthetic-datasets").glob("*") if p.is_dir()]
             synthetic_dataset_list_items = []
             for synthetic_dataset_dir in synthetic_dataset_dirs:
                 synthetic_dataset = read_synthetic_dataset_from_json(synthetic_dataset_dir)
-                if (
-                    not searchTerm
-                    or searchTerm.lower() in (synthetic_dataset.name or "").lower()
-                    or searchTerm.lower() in (synthetic_dataset.description or "").lower()
-                ):
-                    if not status or status == synthetic_dataset.generation_status:
-                        # use model_construct to skip validation and warnings of extra fields
-                        synthetic_dataset_list_items.append(
-                            SyntheticDatasetListItem.model_construct(**synthetic_dataset.model_dump())
-                        )
-            # use jsonable_encoder to handle non-serializable objects like datetime
+                synthetic_dataset_string = " ".join(
+                    [synthetic_dataset.name or "", synthetic_dataset.description or ""]
+                ).lower()
+                if searchTerm and searchTerm.lower() not in synthetic_dataset_string:
+                    continue
+                if status and status != synthetic_dataset.generation_status:
+                    continue
+                # use model_construct to skip validation and warnings of extra fields
+                synthetic_dataset_list_items.append(
+                    SyntheticDatasetListItem.model_construct(**synthetic_dataset.model_dump())
+                )
+
             return JSONResponse(
                 status_code=200,
-                content=jsonable_encoder(
+                content=jsonable_encoder(  # use jsonable_encoder to handle non-serializable objects like datetime
                     {
                         "totalCount": len(synthetic_dataset_list_items),
                         "results": synthetic_dataset_list_items[int(offset) : int(offset) + int(limit)],
@@ -451,8 +626,8 @@ class Routes:
                 reports_dir = self.home_dir / "synthetic-datasets" / id / "ModelQAReports"
             else:
                 reports_dir = self.home_dir / "synthetic-datasets" / id / "DataQAReports"
-            fn = reports_dir / f"{table.name}:{modelType.lower()}.html"
-            return HTMLResponse(content=fn.read_text())
+            fn = reports_dir / f"{get_model_label(table, modelType, path_safe=True)}.html"
+            return HTMLResponse(content=fn.read_text(encoding="utf-8"))
 
         @self.router.get("/synthetic-datasets/{id}/generation", response_model=JobProgress)
         async def get_generation_progress(id: str) -> JobProgress:
